require("levelsv2")
require("cars")
require("saucer")
require("utils")
require("menus")
require("tutorial")
require("UI")

function love.load()
---------LOAD IMAGES
	saucerImage = love.graphics.newImage("assets/vehicles/saucer.png")
	truckImage = love.graphics.newImage("assets/vehicles/pickup.png")
	cropCircleImg = love.graphics.newImage("assets/tile9.png")
	fogImage = love.graphics.newImage("assets/fog.png")
	menuBg = love.graphics.newImage("assets/mainMenuBG.png")
	menuPlayButton = love.graphics.newImage("assets/playButton.png")
	menuPlayButtonLit = love.graphics.newImage("assets/playButtonLit.png")
	menuQuitButton = love.graphics.newImage("assets/quitButton.png")
	menuQuitButtonLit = love.graphics.newImage("assets/quitButtonLit.png")
	moveTut = love.graphics.newImage("assets/moveTut.png")
	beamTut = love.graphics.newImage("assets/beamTut.png")
	mainUI = love.graphics.newImage("assets/mainUI.png")
	UIpanel = love.graphics.newImage("assets/UIpanel.png")
	splash = love.graphics.newImage("assets/instSplash.png")
	endSplash = love.graphics.newImage("assets/gameOver.png")

--	love.graphics.setNewFont("assets/SurfsUp.ttf", 14)

--------INIT TABLES WITH RELATED VARIABLES
	tile = {}
	for i=1,11 do
		tile[i] = love.graphics.newImage( "assets/tile"..i..".png")
	end
	hero = {}
		hero.x = 512	
		hero.y = 600
		hero.speed = 100
		hero.width = saucerImage:getWidth()
		hero.height = saucerImage:getHeight()
		hero.mode = flying
		hero.visibility = 0
		hero.score = 0
	beam = {}
		beam.startX = hero.x
		beam.startY = hero.y
		beam.targetX = beam.startX
		beam.targetY = beam.startY	
		love.graphics.setLineStyle ("rough")
	beamTarget = {}
		beamTarget.x,beamTarget.y = hero.x,hero.y
	flyingCar = {}
	cullCars = {}
	cars = {}
	cropCircles = {}
	wrecks = {}
	capturedCars = {}

--------INIT GLOBAL VARIABLES
	targetX = 0
	targetY = 0
	angle = 0
	sightRange = 128
	beamRange = 256
	pickupRange = 16
	beamSpeed = 50
	timerReset = 5
	newCarTimer = timerReset	
	tileWidth = 32
	tileHeight = 32
	maxLevel = 1
	screenWidth = 1024
	screenHeight = 768
	sightScore = 3
	carSightRange = 48
	circleValue = 5 -- value of drawing one crop circle square on your overall visibility score
	pauseTimer = 2 -- how long to pause for giving a tutorial instruction
	maxVisibility = 5000
	boxHeight = 15
	UITimer = 3
	capValue = 500	
	
--------LOAD SOUND FILES
	bgMusic = love.audio.newSource("assets/sounds/music.ogg")
		bgMusic:setLooping(true)
		bgMusic:setVolume(0.8)
		bgMusic:play()
	
	beamSound = love.audio.newSource("assets/sounds/beam.ogg", "static")
		beamSound:setLooping(true)
		beamSound:setVolume(0.75)
	
--------OTHER LOADING FUNCTIONS
	loadLevels()
	initMenu()
	startTutorial()
	gameState = "menus"
	
end

function love.update(dt)
	if gameState == "menus" then
		updateMenu(dt)
	elseif gameState == "paused" then
		if pauseTimer > 0 then
			pauseTimer = pauseTimer - dt
		else
			gameState = "playing"
			pauseTimer = 3
		end
	elseif gameState == "playing" then
		angle = angle + dt/50
		updateCars(dt) 
		updateSaucer(dt)
		updateTutorial(dt)
		updateUI(dt)
		if mouseCooldown > 0 then
			mouseCooldown = math.max((mouseCooldown - dt), 0)
		end
	end
end

function love.draw()
	if gameState == "menus" then
		drawMenu()
	elseif gameState == "lost" then
		love.graphics.draw(endSplash, 0, 0)
	elseif gameState == "start" then
		love.graphics.draw(splash, 0, 0)
	else
		drawMap()
		drawCars()
		drawCircles()	
		drawFog()
		drawTutorial()
		drawUI()

		if mouseCooldown == 0 then 
			if love.mouse.isDown("l") then
				love.graphics.setLineWidth(7)
				love.graphics.setColor(190,250,210)
				if distance(beamTarget, hero) < beamRange then
					love.graphics.line(beam.startX, beam.startY, beamTarget.x, beamTarget.y)
					love.graphics.setLineWidth(4)
					love.graphics.setColor(0,110,40)
					love.graphics.line(beam.startX, beam.startY, beamTarget.x, beamTarget.y)
				end
				love.graphics.setColor(255,255,255)
			end
		end

		local rot = angle*180/math.pi
		love.graphics.draw(saucerImage, hero.x, hero.y, rot, 1, 1, hero.width/2, hero.height/2)

	end
end

function love.mousereleased(button)
	if gameState == "playing" then
		if button == "l" then
			beamTarget.x = beam.startX
			beamTarget.y = beam.startY
		end
	end
end

function love.mousepressed(x,y,button)
	if gameState == "menus" then
		if checkMouseOver(startButton.x, startButton.y, startButton.width, startButton.height, x, y) then
			if button == "l" then
				love.graphics.clear()
				mouseCooldown = 0.5
				gameState = "start"
			end
		elseif checkMouseOver(quitButton.x, quitButton.y, quitButton.width, quitButton.height, x, y) then
			love.event.push("quit")
		end
	elseif gameState == "start" then
		if button == "l" then
			love.graphics.clear()
			mouseCooldown = 0.5
			gameState = "playing"
		end
	elseif gameState == "lost" then
		if button == "l" then
			love.graphics.clear()
			gameState = "menus"
		end
	end
end

function love.keypressed(key)
	if gameState == "start" then
		love.graphics.clear()
		mouseCooldown = 0.5
		gameState = "playing"
	elseif  gameState == "lost" then
		love.graphics.clear()
		gameState = "menus"
	end
end




